VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CDebug"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit


Private Const MyName As String = "CDebug"

'*******************************************************************************
'******************               Interfaces               *********************
'*******************************************************************************
Implements IDebug

'*******************************************************************************
'******************                 Events                 *********************
'*******************************************************************************

'*******************************************************************************
'******************           Constants, Types             *********************
'*******************************************************************************
Private Const c_CLSName As String = "!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"

'*******************************************************************************
'******************            API-Declarations            *********************
'*******************************************************************************
Private Declare Sub OutputDebugString Lib "kernel32" Alias _
  "OutputDebugStringA" ( _
      ByVal lpOutputString As String _
    )
    
'*******************************************************************************
'******************              Declaration               *********************
'*******************************************************************************

'--- Theseare the different prefix strings used for printouts,
'    set within Sub Init
Private m_PreFix        As String
Private m_TracePre      As String
Private m_TraceInPre    As String
Private m_TraceOutPre   As String
Private m_ErrorPre      As String
Private m_WarnPre       As String

'--- File handling
Private m_sLogFile      As String
Private m_iDatNr        As Integer
Private m_boWriteFile   As Boolean

'--- Indicates if launched from exe (not from IDE)
Private m_FromExe       As Boolean

'--- Stops within Sub TraceError, if set (and launched from IDE)
Private m_StopOnError   As Boolean

'*******************************************************************************
'******************                 Class                  *********************
'*******************************************************************************
Private Sub Class_Initialize()

  m_FromExe = False
  On Error GoTo FromIde
    Debug.Assert 1 / 0
    m_FromExe = True
    
FromIde:
  Resume GoOn 'Clears Error
GoOn:
  
  On Error Resume Next
    
  '--- launc DbgView automatically if in App.-path
  CheckDebugView
  
  m_sLogFile = App.Path & "\" & App.EXEName & ".txt"
  m_iDatNr = FreeFile
  
  ' Ide = Not FromExe
  'Exist    Ide    Kill Output Append
  '----------------------------------
  '  0       0  |   0     1      0
  '  0       1  |   0     1      0
  '  1       0  |   0     0      1
  '  1       1  |   1     1      0
  If FileExists(m_sLogFile) Then
    If Not FromExe Then
      Kill m_sLogFile
      Open m_sLogFile For Output As m_iDatNr
    Else
      Open m_sLogFile For Append As m_iDatNr
    End If
  Else
    Open m_sLogFile For Output As m_iDatNr
  End If

End Sub

Private Sub Class_Terminate()
  On Error Resume Next
  Close m_iDatNr
End Sub

'*******************************************************************************
'******************                Privates                *********************
'*******************************************************************************

'--- Format function name
'
Private Function FCLSName(ByVal vName As String) As String

  FCLSName = Format$(vName, c_CLSName)

End Function

'--- Write to different targets
'
Private Sub DebugOutput( _
    ByVal Data As String _
  )
  
  Debug.Print Data
  OutputDebugString Data
  If m_boWriteFile Then
    PrintMsg Data
  End If

End Sub

'--- Write to file
'
Private Sub PrintMsg( _
    ByVal msg As String _
  )

  On Error Resume Next

  Write #m_iDatNr, msg
  Close m_iDatNr                            'Flushes buffer
  Open m_sLogFile For Append As m_iDatNr
  
End Sub

'--- FromExe indicates that app was not started from IDE
'
Private Property Get FromExe() As Boolean
  FromExe = m_FromExe
End Property


'*******************************************************************************
'******************               Interface                *********************
'******************                iDebug                  *********************
'*******************************************************************************


'***Public**********************************************************************
'
'  Procedure:  Sub IDebug_Init
'
' Parameters:  -
'
'     Return:
'
'Description:  IDebug_Init is blabla
'
'Side effects:
'
'*******************************************************************************
Private Sub IDebug_Init( _
    ByVal vPreFix As String, _
    ByVal vStopOnError As Boolean, _
    ByVal vWriteToFile As Boolean _
  )
  
  m_boWriteFile = vWriteToFile
  m_StopOnError = vStopOnError
  m_PreFix = vPreFix
  
  If Not Right$(m_PreFix, 1) = "." Then
  
    m_PreFix = m_PreFix & "."
  
  End If
  
  m_TracePre = "Trace: " & m_PreFix
  m_TraceInPre = ">>>>>: " & m_PreFix
  m_TraceOutPre = "<<<<<: " & m_PreFix
  m_ErrorPre = "ERROR: " & m_PreFix
  m_WarnPre = "Warn:  " & m_PreFix
    
End Sub

'--- If True, printouts are also routed to a file
'
Private Property Let IDebug_WriteToFile( _
    ByVal vEnable As Boolean _
  )

  m_boWriteFile = vEnable

End Property

'--- Status of the flag
'
Private Property Get IDebug_WriteToFile() As Boolean
  IDebug_WriteToFile = m_boWriteFile
End Property

'--- IF True ==> Stops in function IDebug_TraceError,
'                when run from IDE
'
Private Property Let IDebug_StopOnError( _
    ByVal vStop As Boolean _
  )
  m_StopOnError = vStop
End Property

'--- Status of the flag
'
Private Property Get IDebug_StopOnError() As Boolean
  IDebug_StopOnError = m_StopOnError
End Property

'--- This clears the DbgView.exe
'
Private Sub IDebug_TraceClear()
  IDebug_TraceThrough "DBGVIEWCLEAR"
End Sub

'--- Print a msg as trace in, which means it is prefixed by the
'    trace in string passed by sub init
'
Private Function IDebug_TraceIn( _
    ByVal sFunction As String, _
    ByVal msg As String _
  ) As Boolean
  
  Dim tmpStr As String
  tmpStr = m_TraceInPre & FCLSName(sFunction) & ": " & msg
  DebugOutput tmpStr
  
  IDebug_TraceIn = True
  
End Function

'--- Print a msg as trace out, which means it is prefixed by the
'    trace out string passed by sub init
'
Private Function IDebug_TraceOut( _
    ByVal sFunction As String, _
    ByVal msg As String _
  ) As Boolean
  
  Dim tmpStr As String
  tmpStr = m_TraceOutPre & FCLSName(sFunction) & ": " & msg
  DebugOutput tmpStr
  
  IDebug_TraceOut = True
  
End Function

'--- Print a msg as trace, which means it is prefixed by the
'    trace string passed by sub init
'
Private Function IDebug_Trace( _
    ByVal sFunction As String, _
    ByVal msg As String _
  ) As Boolean
  
  Dim tmpStr As String
  tmpStr = m_TracePre & FCLSName(sFunction) & ": " & msg
  DebugOutput tmpStr
  
  IDebug_Trace = True
  
End Function

'--- Print a msg as error, which means it is prefixed by the
'    error string passed by sub init
'    An error log could be invoked here.
'
Private Function IDebug_TraceError( _
    ByVal sFunction As String, _
    ByVal msg As String, _
    Optional ByVal vPrintErr As Boolean = False _
  ) As Boolean

  Dim tmpStr    As String
  Dim errInfo   As String
  
  tmpStr = m_ErrorPre & FCLSName(sFunction) & ": "
  
  errInfo = vbNullString
  If vPrintErr Then
    errInfo = vbNewLine & Space(Len(tmpStr)) & CStr(Err.Number) & ": " & Err.Description
  End If
  
  tmpStr = tmpStr & msg & errInfo
  DebugOutput tmpStr
  
  IDebug_TraceError = m_StopOnError
  Debug.Assert IDebug_TraceError

End Function

'--- Print a msg as warning, which means it is prefixed by the
'    warning string passed by sub init
'
Private Function IDebug_TraceWarn( _
    ByVal sFunction As String, _
    ByVal msg As String _
  ) As Boolean
  
  Dim tmpStr As String
  tmpStr = m_WarnPre & FCLSName(sFunction) & ": " & msg
  DebugOutput tmpStr
  
  IDebug_TraceWarn = True

End Function

'--- Printout msg without any modifications
'
Private Sub IDebug_TraceThrough( _
    ByVal msg As String _
  )
  
  DebugOutput msg

End Sub

